#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module defines various general reusable data
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

WORLD_COUNTRIES = (

("AF", "AFGHANISTAN"),
("AX", "ÅLAND ISLANDS"),
("AL", "ALBANIA"),
("DZ", "ALGERIA"),
("AS", "AMERICAN SAMOA"),
("AD", "ANDORRA"),
("AO", "ANGOLA"),
("AI", "ANGUILLA"),
("AQ", "ANTARCTICA"),
("AG", "ANTIGUA AND BARBUDA"),
("AR", "ARGENTINA"),
("AM", "ARMENIA"),
("AW", "ARUBA"),
("AU", "AUSTRALIA"),
("AT", "AUSTRIA"),
("AZ", "AZERBAIJAN"),
("BS", "BAHAMAS"),
("BH", "BAHRAIN"),
("BD", "BANGLADESH"),
("BB", "BARBADOS"),
("BY", "BELARUS"),
("BE", "BELGIUM"),
("BZ", "BELIZE"),
("BJ", "BENIN"),
("BM", "BERMUDA"),
("BT", "BHUTAN"),
("BO", "BOLIVIA"),
("BA", "BOSNIA AND HERZEGOVINA"),
("BW", "BOTSWANA"),
("BV", "BOUVET ISLAND"),
("BR", "BRAZIL"),
("IO", "BRITISH INDIAN OCEAN TERRITORY"),
("BN", "BRUNEI DARUSSALAM"),
("BG", "BULGARIA"),
("BF", "BURKINA FASO"),
("BI", "BURUNDI"),
("KH", "CAMBODIA"),
("CM", "CAMEROON"),
("CA", "CANADA"),
("CV", "CAPE VERDE"),
("KY", "CAYMAN ISLANDS"),
("CF", "CENTRAL AFRICAN REPUBLIC"),
("TD", "CHAD"),
("CL", "CHILE"),
("CN", "CHINA"),
("CX", "CHRISTMAS ISLAND"),
("CC", "COCOS (KEELING) ISLANDS"),
("CO", "COLOMBIA"),
("KM", "COMOROS"),
("CG", "REPUBLIC OF CONGO"),
("CD", "DR CONGO"),
("CK", "COOK ISLANDS"),
("CR", "COSTA RICA"),
("CI", "CÔTE D'IVOIRE"),
("HR", "CROATIA"),
("CU", "CUBA"),
("CY", "CYPRUS"),
("CZ", "CZECH REPUBLIC"),
("DK", "DENMARK"),
("DJ", "DJIBOUTI"),
("DM", "DOMINICA"),
("DO", "DOMINICAN REPUBLIC"),
("EC", "ECUADOR"),
("EG", "EGYPT"),
("SV", "EL SALVADOR"),
("GQ", "EQUATORIAL GUINEA"),
("ER", "ERITREA"),
("EE", "ESTONIA"),
("ET", "ETHIOPIA"),
("FK", "FALKLAND ISLANDS (MALVINAS)"),
("FO", "FAROE ISLANDS"),
("FJ", "FIJI"),
("FI", "FINLAND"),
("FR", "FRANCE"),
("GF", "FRENCH GUIANA"),
("PF", "FRENCH POLYNESIA"),
("TF", "FRENCH SOUTHERN TERRITORIES"),
("GA", "GABON"),
("GM", "GAMBIA"),
("GE", "GEORGIA"),
("DE", "GERMANY"),
("GH", "GHANA"),
("GI", "GIBRALTAR"),
("GR", "GREECE"),
("GL", "GREENLAND"),
("GD", "GRENADA"),
("GP", "GUADELOUPE"),
("GU", "GUAM"),
("GT", "GUATEMALA"),
("GG", "GUERNSEY"),
("GN", "GUINEA"),
("GW", "GUINEA-BISSAU"),
("GY", "GUYANA"),
("HT", "HAITI"),
("HM", "HEARD ISLAND AND MCDONALD ISLANDS"),
("VA", "HOLY SEE (VATICAN CITY STATE)"),
("HN", "HONDURAS"),
("HK", "HONG KONG"),
("HU", "HUNGARY"),
("IS", "ICELAND"),
("IN", "INDIA"),
("ID", "INDONESIA"),
("IR", "IRAN"),
("IQ", "IRAQ"),
("IE", "IRELAND"),
("IM", "ISLE OF MAN"),
("IL", "ISRAEL"),
("IT", "ITALY"),
("JM", "JAMAICA"),
("JP", "JAPAN"),
("JE", "JERSEY"),
("JO", "JORDAN"),
("KZ", "KAZAKHSTAN"),
("KE", "KENYA"),
("KI", "KIRIBATI"),
("KP", "KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF"),
("KR", "KOREA, REPUBLIC OF"),
("KW", "KUWAIT"),
("KG", "KYRGYZSTAN"),
("LA", "LAO PEOPLE'S DEMOCRATIC REPUBLIC"),
("LV", "LATVIA"),
("LB", "LEBANON"),
("LS", "LESOTHO"),
("LR", "LIBERIA"),
("LY", "LIBYAN ARAB JAMAHIRIYA"),
("LI", "LIECHTENSTEIN"),
("LT", "LITHUANIA"),
("LU", "LUXEMBOURG"),
("MO", "MACAO"),
("MK", "MACEDONIA"),
("MG", "MADAGASCAR"),
("MW", "MALAWI"),
("MY", "MALAYSIA"),
("MV", "MALDIVES"),
("ML", "MALI"),
("MT", "MALTA"),
("MH", "MARSHALL ISLANDS"),
("MQ", "MARTINIQUE"),
("MR", "MAURITANIA"),
("MU", "MAURITIUS"),
("YT", "MAYOTTE"),
("MX", "MEXICO"),
("FM", "MICRONESIA"),
("MD", "MOLDOVA"),
("MC", "MONACO"),
("MN", "MONGOLIA"),
("ME", "MONTENEGRO"),
("MS", "MONTSERRAT"),
("MA", "MOROCCO"),
("MZ", "MOZAMBIQUE"),
("MM", "MYANMAR"),
("NA", "NAMIBIA"),
("NR", "NAURU"),
("NP", "NEPAL"),
("NL", "NETHERLANDS"),
("AN", "NETHERLANDS ANTILLES"),
("NC", "NEW CALEDONIA"),
("NZ", "NEW ZEALAND"),
("NI", "NICARAGUA"),
("NE", "NIGER"),
("NG", "NIGERIA"),
("NU", "NIUE"),
("NF", "NORFOLK ISLAND"),
("MP", "NORTHERN MARIANA ISLANDS"),
("NO", "NORWAY"),
("OM", "OMAN"),
("PK", "PAKISTAN"),
("PW", "PALAU"),
("PS", "PALESTINIAN TERRITORY"),
("PA", "PANAMA"),
("PG", "PAPUA NEW GUINEA"),
("PY", "PARAGUAY"),
("PE", "PERU"),
("PH", "PHILIPPINES"),
("PN", "PITCAIRN"),
("PL", "POLAND"),
("PT", "PORTUGAL"),
("PR", "PUERTO RICO"),
("QA", "QATAR"),
("RE", "REUNION"),
("RO", "ROMANIA"),
("RU", "RUSSIAN FEDERATION"),
("RW", "RWANDA"),
("BL", "SAINT BARTHÉLEMY"),
("SH", "SAINT HELENA"),
("KN", "SAINT KITTS AND NEVIS"),
("LC", "SAINT LUCIA"),
("MF", "SAINT MARTIN"),
("PM", "SAINT PIERRE AND MIQUELON"),
("VC", "SAINT VINCENT AND THE GRENADINES"),
("WS", "SAMOA"),
("SM", "SAN MARINO"),
("ST", "SAO TOME AND PRINCIPE"),
("SA", "SAUDI ARABIA"),
("SN", "SENEGAL"),
("RS", "SERBIA"),
("SC", "SEYCHELLES"),
("SL", "SIERRA LEONE"),
("SG", "SINGAPORE"),
("SK", "SLOVAKIA"),
("SI", "SLOVENIA"),
("SB", "SOLOMON ISLANDS"),
("SO", "SOMALIA"),
("ZA", "SOUTH AFRICA"),
("GS", "SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS"),
("ES", "SPAIN"),
("LK", "SRI LANKA"),
("SD", "SUDAN"),
("SR", "SURINAME"),
("SJ", "SVALBARD AND JAN MAYEN"),
("SZ", "SWAZILAND"),
("SE", "SWEDEN"),
("CH", "SWITZERLAND"),
("SY", "SYRIAN ARAB REPUBLIC"),
("TW", "TAIWAN"),
("TJ", "TAJIKISTAN"),
("TZ", "TANZANIA"),
("TH", "THAILAND"),
("TL", "TIMOR-LESTE"),
("TG", "TOGO"),
("TK", "TOKELAU"),
("TO", "TONGA"),
("TT", "TRINIDAD AND TOBAGO"),
("TN", "TUNISIA"),
("TR", "TURKEY"),
("TM", "TURKMENISTAN"),
("TC", "TURKS AND CAICOS ISLANDS"),
("TV", "TUVALU"),
("UG", "UGANDA"),
("UA", "UKRAINE"),
("AE", "UNITED ARAB EMIRATES"),
("GB", "UNITED KINGDOM"),
("US", "UNITED STATES"),
("UM", "UNITED STATES MINOR OUTLYING ISLANDS"),
("UY", "URUGUAY"),
("UZ", "UZBEKISTAN"),
("VU", "VANUATU"),
("VE", "VENEZUELA"),
("VN", "VIET NAM"),
("VG", "BRITISH VIRGIN ISLANDS"),
("VI", "U.S VIRGIN ISLANDS"),
("WF", "WALLIS AND FUTUNA"),
("EH", "WESTERN SAHARA"),
("YE", "YEMEN"),
("ZM", "ZAMBIA"),
("ZW", "ZIMBABWE"),

)

# Offset from UTC in seconds, and human readable name
WORLD_TIME_ZONES = (

("-43200", "UTC -12:00"),
("-39600", "UTC -11:00"),
("-36000", "UTC -10:00"),
("-34200", "UTC -9:30"),
("-32400", "UTC -9:00"),
("-28800", "UTC -8:00"),
("-25200", "UTC -7:00"),
("-21600", "UTC -6:00"),
("-18000", "UTC -5:00"),
("-16200", "UTC -4:30"),
("-14400", "UTC -4:00"),
("-12600", "UTC -3:30"),
("-10800", "UTC -3:00"),
("-7200", "UTC -2:00"),
("-3600", "UTC -1:00"),
("0", "UTC +0:00"),
("3600", "UTC +1:00"),
("7200", "UTC +2:00"),
("10800", "UTC +3:00"),
("12600", "UTC +3:30"),
("14400", "UTC +4:00"),
("16200", "UTC +4:30"),
("18000", "UTC +5:00"),
("19800", "UTC +5:30"),
("20700", "UTC +5:45"),
("21600", "UTC +6:00"),
("23400", "UTC +6:30"),
("25200", "UTC +7:00"),
("28800", "UTC +8:00"),
("31500", "UTC +8:45"),
("32400", "UTC +9:00"),
("34200", "UTC +9:30"),
("36000", "UTC +10:00"),
("38700", "UTC +10:30"),
("39600", "UTC +11:00"),
("41400", "UTC +11:30"),
("43200", "UTC +12:00"),
("45600", "UTC +12:40"),
("46800", "UTC +13:00"),
("50400", "UTC +14:00"),

)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Affandi Mustapa <affandi@marimore.co.jp>
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Paypal billing module

Contains classes and functions to process Paypal requests
G{importgraph}
"""

__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

from mamopublic import common
import urllib, urllib2

from mamopublic.utils.helpers import log_syslog

class PaypalAPI(common.BaseClass):
    """
    This class enacapulates utilities to verify and use IPN messages from Paypal
    """

    _PP_CALLBACK_TEST_URL   = "https://www.sandbox.paypal.com/cgi-bin/webscr"
    _PP_CALLBACK_URL        = "https://www.paypal.com/cgi-bin/webscr"

    def __init__(self, ipn_data, config={}):
        """

        @type ipn_data: dictionary
        @param ipn_data: The ipn data i.e django's request.POST.copy() 
        """

        # Configurations are passed to the class but this class should
        # not be connected to django directly
        self._config = config
        self._ipn_data = ipn_data

        if self._config.get('paypal_prod_flag', False):
            self._test_flag = False
        else:
            # If flag is not set, default is in testing
            self._test_flag = True

    def verify_ipn(self):
        """
        Verifies that the IPN is authentic

        @return: True if the paypal ipn was verified succesfully
        """

        if self._test_flag:
            _PP_URL = self._PP_CALLBACK_TEST_URL
        else:
            _PP_URL = self._PP_CALLBACK_URL

        ipn_data = self._ipn_data
        ipn_data['cmd'] = "_notify-validate"

        tmp_ipn_data = {}
        for k in ipn_data.keys():
            tmp_ipn_data[k] = ipn_data[k].encode(ipn_data['charset'])

        params = urllib.urlencode(tmp_ipn_data)

        req = urllib2.Request(_PP_URL, params)
        req.add_header("Content-type", "application/x-www-form-urlencoded")

        try:
            response = urllib2.urlopen(req)
            status = response.read().strip()
        except IOError:
            return -100

        if status == "VERIFIED":
            return 0

        return -1

    def check_seller_email(self):

        if self._ipn_data['receiver_email'] == self._config['paypal_seller_email']:
            return True

        return False




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Utilities used for debugging during development
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

import datetime, tempfile

def log(str, fn="mamodebug.log"):
    """
    Writes an arbitary string to a file called 'mamodebug.log' by default.
    The log file will be placed in the directory returned by tempfile.gettempdir()

    @type str: string
    @param str: The string for output
    @type fn: string
    @param fn: The filename for the log file. Defaults to mamodebug.log. 
               This file WILL be under the directory returned by tempfile.gettempdir()
    """

    now = datetime.datetime.strftime(datetime.datetime.now(), "%Y-%m-%d %H:%M:%S")
    dir = tempfile.gettempdir()

    fd = open("%s/%s" % (dir, fn), 'a+')
    fd.write("[%s] %s\n" % (now, str))
    fd.close()


def logger(str, fn):
    """
    Writes an arbitary string to a file with the path fn

    @type str: string
    @param str: The string for output
    @type fn: string
    @param fn: The file path for the log
    """

    now = datetime.datetime.strftime(datetime.datetime.now(), "%Y-%m-%d %H:%M:%S")

    fd = open("%s" % (fn), 'a+')
    fd.write("[%s] %s\n" % (now, str))
    fd.close()


if __name__ == '__main__':

    log("Hello World! 漢次テストあいうえお")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module defines commonly used code for the mamo package
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

import re

class BaseClass(object):
    """
    BaseClass contains very common functions
    and implementations for mamo classes. All of mamo packages classes
    uses BaseClass as their parent class
    """

    def read_only_property(self):
        raise (AttributeError), "Read-only attribute"

    def _prop_set_classname(self, value):
        self.read_only_property()

    def _prop_get_classname(self):
        compiled_re = re.compile("'.*'")
        clsname = compiled_re.search("%s" % (self.__class__)).group()
        clsname = clsname.replace("'","")
        clsname = clsname.replace("%s" % (self.__module__), "")
        clsname = clsname.replace(".","")
        return clsname

    myclassname = property(_prop_get_classname, _prop_set_classname, 
                           doc="Returns the name of the class")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module defines commonly used utility code for working with files
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

import random
import time
import string

def generate_random_filename(randlen=2, suffix=None):
    """
    Generates a random string with a specified prefix, useful for generating
    random filenames. No two filenames will be the same if executed on the same machine

    @type  suffix: string or None
    @param suffix: The suffix to append at the end of the filename, i.e xxxx.gif or xxxx.html
                   Default is None.
    @type  randlen: integer
    @param  randlen: The length of the random prefix for the filename. Defaults to 2 with a maximum of 7
    @rtype: string
    @return: The random filename
    """

    try:
        int(randlen)
        if randlen > 7:
            randlen = 7
    except:
        randlen = 2

    if suffix:
        return "%s%s.%s" % ( "".join(random.sample(string.digits + string.ascii_lowercase, randlen)), int(time.time()), suffix.strip() )
    else:
        return "%s%s" % ( "".join(random.sample(string.digits + string.ascii_lowercase, randlen)), int(time.time()) )


if __name__ == '__main__':

    print generate_random_filename()
    print generate_random_filename("jpg")
    print generate_random_filename(7,"jpg")
    print generate_random_filename(10,"jpg")
    print generate_random_filename(5,"jpg")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module defines helper functions
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

import sys, datetime
import inspect

def import_object(qualified_name):
    """
    import_object() will return the module/function/class which is specified
    in qualified_name

    @type qualified_name: string
    @param qualified_name: The fully qualified package path to the target
    @rtype: object or None
    @return: An object to the target module/function/class or None if an exception
             occurred
    """

    parent_namespace = ".".join(qualified_name.split(".")[:-1])
    target_namespace = qualified_name.split(".")[-1]

    if parent_namespace == "":
        # This is the top module
        parent_namespace = target_namespace

    try:
        __import__(parent_namespace)
        m = sys.modules[parent_namespace]
        mod = getattr(m, target_namespace)
        return mod

    except (Exception) as e:
        print "Exception occurred 02: import_object(): %s" % (e)
        return None


def log_syslog(ident, message, priority="LOG_NOTICE", facility="LOG_USER"):
    """
    Writes log messages via the system syslog(). This is for UNIX based systems only.

    @type facility: string
    @param facility: The facility to be used. It can be one of these:
        1. LOG_KERN
        2. LOG_USER
        3. LOG_MAIL
        4. LOG_DAEMON
        5. LOG_AUTH
        6. LOG_LPR
        7. LOG_NEWS
        8. LOG_UUCP
        9. LOG_CRON 
        10. LOG_LOCAL0 to LOG_LOCAL7
    By default it will be LOG_USER

    @type priority: string
    @param priority: The priority of the message, which can be one of these:
        1. LOG_EMERG
        2. LOG_ALERT
        3. LOG_CRIT
        4. LOG_ERR
        5. LOG_WARNING
        6. LOG_NOTICE
        7. LOG_INFO
        8. LOG_DEBUG
    By default it is LOG_NOTICE

    @type ident: string
    @param ident: The prepended indent for your message. Usually this is the filename.
    @type message: string
    @param message: The message you want to log to
    """

    try:
        import syslog
    except:
        return -1

    if facility == "LOG_KERN":
        fac = syslog.LOG_KERN
    elif facility == "LOG_MAIL":
        fac = syslog.LOG_MAIL
    elif facility == "LOG_DAEMON":
        fac = syslog.LOG_DAEMON
    elif facility == "LOG_AUTH":
        fac = syslog.LOG_AUTH
    elif facility == "LOG_LPR":
        fac = syslog.LOG_LPR
    elif facility == "LOG_NEWS":
        fac = syslog.LOG_NEWS
    elif facility == "LOG_UUCP":
        fac = syslog.LOG_UUCP
    elif facility == "LOG_CRON":
        fac = syslog.LOG_CRON
    elif facility[:9] == "LOG_LOCAL":
        if facility[9] == "0":
            fac = syslog.LOG_LOCAL0
        elif facility[9] == "1":
            fac = syslog.LOG_LOCAL1
        elif facility[9] == "2":
            fac = syslog.LOG_LOCAL2
        elif facility[9] == "3":
            fac = syslog.LOG_LOCAL3
        elif facility[9] == "4":
            fac = syslog.LOG_LOCAL4
        elif facility[9] == "5":
            fac = syslog.LOG_LOCAL5
        elif facility[9] == "6":
            fac = syslog.LOG_LOCAL6
        elif facility[9] == "7":
            fac = syslog.LOG_LOCAL7
        else:
            # There's only 0 to 7
            fac = syslog.LOG_USER
    else:
        fac = syslog.LOG_USER

    if priority == "LOG_EMERG":
        prio = syslog.LOG_EMERG
    elif priority == "LOG_ALERT":
        prio = syslog.LOG_ALERT
    elif priority == "LOG_CRIT":
        prio = syslog.LOG_CRIT
    elif priority == "LOG_ERR":
        prio = syslog.LOG_ERR
    elif priority == "LOG_WARNING":
        prio = syslog.LOG_WARNING
    elif priority == "LOG_INFO":
        prio = syslog.LOG_INFO
    elif priority == "LOG_DEBUG":
        prio = syslog.LOG_DEBUG
    else:
        prio = syslog.LOG_NOTICE

    syslog.openlog(ident, 0, fac)
    syslog.syslog(prio, message)

def log_syslogn(message, ident=None, priority="LOG_NOTICE", facility="LOG_USER"):
    """
    Shortcut to log to LOCALn. The function signature should be similar to log_syslog
    except that message is swapped to be the first so indent would be optional.
    If indent is None, use inspect module to get the calling function name.
    """
    def get_ident(frame):
        frame_info = inspect.getframeinfo(frame[0])

        if 'self' in frame[0].f_locals:
            instance = frame[0].f_locals['self']
        else:
            instance = None

        mod = inspect.getmodule(frame[0])
        filename, lineno, function, code_context, index = frame_info

        if instance:
            ident = '%s.%s.%s()' % (mod.__name__, instance.__class__.__name__,
                                    function.strip())
        else:
            if mod.__name__ == '__main__':
                modname = filename
            else:
                modname = mod.__name__
            ident = '%s.%s()' % (modname, function.strip())

        return ident

    if not ident:
        frame = None
        frame_info = None
        try:
            frame = inspect.stack()[2]
            ident = get_ident(frame)
        except Exception:
            ident = ''
        finally:
            del frame
            del frame_info

    # split message by newline and then joining it back, removing any tabs or
    # space that resulted from indentation. Log message always in single line.
    messages = [line.strip() for line in message.split()]
    message1 = " ".join(messages)

    log_syslog(ident, message1, priority, facility=facility) 

def log_syslog0(message, ident=None, priority="LOG_NOTICE", facility="LOG_LOCAL0"):
    log_syslogn(message, ident, priority, facility=facility) 

def log_syslog1(message, ident=None, priority="LOG_NOTICE", facility="LOG_LOCAL1"):
    log_syslogn(message, ident, priority, facility=facility) 

def log_syslog2(message, ident=None, priority="LOG_NOTICE", facility="LOG_LOCAL2"):
    log_syslogn(message, ident, priority, facility=facility) 

def log_syslog3(message, ident=None, priority="LOG_NOTICE", facility="LOG_LOCAL3"):
    log_syslogn(message, ident, priority, facility=facility) 

def log_syslog4(message, ident=None, priority="LOG_NOTICE", facility="LOG_LOCAL4"):
    log_syslogn(message, ident, priority, facility=facility) 

def uniqify_list(seq, idfun=None): 
    """
    Uniqify a list while preserving order. i.e

    If you have a list such as [1,2,2,2,3,4,5,6,6,6,6] uniqify_list() while 
    return [1,2,3,4,5,6]

    You can also specify a function to transform the data, i.e

    >>> a=list('ABeeE')
    >>> uniqify_list(a)
    ['A','B','e','E']
    >>> uniqify_list(a, lambda x: x.lower())
    ['A','B','e'] 

    Taken from http://www.peterbe.com/plog/uniqifiers-benchmark

    @type seq: list
    @param seq: The list which you want to uniqify 
    @type idfun: a function, defaults to None
    @param idfun: A function which you want to process the data with
    @rtype: list
    @return: An list which has been uniqified
    """

    if idfun is None:
        def idfun(x): return x

    seen = {}
    result = []
    for item in seq:
        marker = idfun(item)
        if marker in seen: continue
        seen[marker] = 1
        result.append(item)

    return result

def calculate_time(base_time, diff_seconds, reverse=False): 
    """
    Calculates a new time if you add diff_seconds to base_time

    @type base_time: datetime object
    @param base_time: The base time which you want to do the calculation on
    @type diff_seconds: signed int
    @param diff_seconds: The time difference in seconds. Use a negative value if the time is in the past
    @type reverse: bool
    @param reverse: Reverse calculation; Instead of adding diff_seconds to base_time, you do deduction.
                    This is useful in cases where you have the localtime and the delta but you want to
                    find the UTC.
    @rtype: datetime object
    @return: A datetime object representing the calculated time
    """

    diff_time = datetime.timedelta(seconds=diff_seconds)
    if reverse:
        new_time = base_time - diff_time
    else:
        new_time = base_time + diff_time
    return new_time

if __name__ == '__main__':
    #log_syslog("test", "test test")
    #import_object('test')
    print uniqify_list(list('123123454324332ABCAABCabcaabcrt'))
    print calculate_time(datetime.datetime.now(), -60)
    print calculate_time(datetime.datetime.now(), -60, True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module defines commonly used utility code for working with strings
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

import string

def is_alnum(str, okchars=None):
    """
    Checks if the given string only contains alnum characters (i.e like string.alnum) but allows you
    to also include specific characters which you want to allow

    @type str: string
    @param str: The string we want to check for
    @type okchars: string
    @param okchars: A string of ok characters, besides the normal alnum chars (i.e hyphens)
    @return: True if the given string only contains alnum and allowed characters
    """

    if okchars:
        _oklist = list(string.ascii_letters + string.digits) + list(okchars)
    else:
        _oklist = list(string.ascii_letters + string.digits)

    str_list = list(str)
    for s in str_list:
        if s not in _oklist:
            return False

    return True


if __name__ == '__main__':

    print is_alnum("bauAhfg345%")
    print is_alnum("bauAhfg345%!@", "!@%")
    print is_alnum("bauAhfg345%!@", "!%")
    print is_alnum("bauAhfg345")
    print is_alnum("漢字abc")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Data manupulating utilities for JP mobile 
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate: 2011-01-15 11:22:39 +0900 (Sat, 15 Jan 2011) $"
__version__     = "$LastChangedRevision: 31 $"

def get_mobile_email_domains():
    """
    Returns you a dictionary with lists of recognised domains for emails from mobile

    The dictionary will contain carrier names for keys, and a list of domain names for
    each carrier.

    Possible carrier names are:
        1. DOCOMO
        2. EZWEB
        3. SOFTBANK
        4. WILLCOM
        5. EMOBILE
        6. FREEMAIL (i.e gmail.com used by smartphones)

    The domains listed in this file must:
        1. Have a SPF record 

    @rtype: dictionary
    @returns: A dictionary containing a list of domain names recognised for a 
              particular carrier
    """

    _DOCOMO = (
        'docomo.ne.jp', 
        'mopera.net', 
        'mopera.ne.jp', 
        'docomo.blackberry.com',
        'dwmail.jp',
    )

    _EZWEB = (
        'ezweb.ne.jp', 
    )

    _SOFTBANK = (
        'softbank.ne.jp', 
        'i.softbank.jp', 
        't.vodafone.ne.jp', 
        'd.vodafone.ne.jp', 
        'h.vodafone.ne.jp', 
        'c.vodafone.ne.jp', 
        'k.vodafone.ne.jp', 
        'r.vodafone.ne.jp', 
        'n.vodafone.ne.jp', 
        's.vodafone.ne.jp', 
        'q.vodafone.ne.jp', 
        'jp-c.ne.jp', 
        'jp-d.ne.jp', 
        'jp-h.ne.jp', 
        'jp-k.ne.jp', 
        'jp-n.ne.jp', 
        'jp-r.ne.jp', 
        'jp-s.ne.jp', 
        'jp-t.ne.jp', 
        'jp-q.ne.jp', 
        'disney.ne.jp', 
    )

    _WILLCOM = (
        'willcom.com', 
        'pdx.ne.jp', 
        'di.pdx.ne.jp', 
        'dj.pdx.ne.jp', 
        'dk.pdx.ne.jp', 
        'wm.pdx.ne.jp', 
    )

    _EMOBILE = (
        'emnet.ne.jp',
    )

    _FREEMAIL = (
        'gmail.com', 
    )

    return { 
        'DOCOMO': _DOCOMO,
        'EZWEB': _EZWEB,
        'SOFTBANK': _SOFTBANK,
        'WILLCOM': _WILLCOM,
        'EMOBILE': _EMOBILE,
        'FREEMAIL': _FREEMAIL,
    }


if __name__ == '__main__':

    print get_mobile_email_domains()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE LLC Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE LLC nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Verifying utilities for JP mobile 
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

import pickle

# Use iplib from http://erlug.linux.it/~da/soft/iplib/
from iplib import CIDR

from mamopublic.jp.mobemail import get_mobile_email_domains

IP_DATA_FILE = "../../data/ip.dat"

CONST_HTTP_META_CARRIER    = "HTTP_X_MOGO_CARRIER"
CONST_HTTP_META_BROWSER    = "HTTP_X_MOGO_BROWSER"
CONST_HTTP_META_LOCALE     = "HTTP_X_MOGO_LOCALE"

CONST_SESSION_DATA         = "MOBILE_SESSION"

CONST_IPHONE_UID           = "IPHONE_FIXED"
CONST_ANDROID_UID          = "ANDROID_FIXED"

def is_mobile_ip(ip):
    """
    Checks if an IP is coming from a mobile gateway.

    Possible carrier names are:
        1. DOCOMO
        2. EZWEB
        3. SOFTBANK
        4. WILLCOM

    Possible browsers:
        1. MOBILE
        2. PC

    @type ip: string
    @param ip: The IP address of a client
    @rtype: tuple
    @return: A four element tuple with the first element being a boolean True if 
             the IP is from a mobile gateway, the second is a string with
             the carrier name, the third a string with the browser type 
             and the fourth will be a string with the locale info. 
             If ip is not a mobile ip, all string will be set to None.
             i.e (True, 'DOCOMO', 'MOBILE', 'JP'), (False, None, None, None) or
             (True, 'SOFTBANK', 'PC', 'MY')
    """
    try:
        fd = open(mogodef.MOGO_IP_DATA_FILE, 'rb')
        ipdata_list = pickle.load(fd)
        fd.close()

        for ipdata in ipdata_list:
            iprange = ipdata['ip']
            locale  = ipdata['locale']
            browser = ipdata['browser']
            carrier = ipdata['carrier']

            cidr = CIDR(iprange)
            if cidr.is_valid_ip(ip):
                return (True, carrier, browser, locale)

    except Exception, e:
        pass

    return (False, None, None, None)

def is_mobile_ua(ua):
    """
    Checks if an agent is a mobile.

    Possible carrier names are:
        1. DOCOMO
        2. EZWEB
        3. SOFTBANK
        4. WILLCOM

    Possible browsers:
        1. MOBILE
        2. PC

    @type ua: string
    @param ua: The IP address of a client
    @rtype: tuple
    @return: A four element tuple with the first element being a boolean True if 
             the IP is from a mobile gateway, the second is a string with
             the carrier name, the third a string with the browser type 
             and the fourth will be a string with the locale info. 
             If ip is not a mobile ip, all string will be set to None.
             i.e (True, 'DOCOMO', 'MOBILE', 'JP'), (False, None, None, None) or
             (True, 'SOFTBANK', 'PC', 'MY')
    """

    _ua_dict = {
        "DoCoMo/"       :  "DOCOMO",
        "Docomo"        :  "DOCOMO",
        "HT-03A"        :  "DOCOMO",
        "SO-01B"        :  "DOCOMO",
        "UP.Browser"    :  "EZWEB",
        "KDDI-"         :  "EZWEB",
        "J-PHONE"       :  "SOFTBANK",
        "Vodafone"      :  "SOFTBANK",
        "MOT-"          :  "SOFTBANK",
        "SoftBank"      :  "SOFTBANK",
        "iPhone"        :  "SOFTBANK",
        "HTC"           :  "SOFTBANK",
        "MOTEMULATOR"   :  "SOFTBANK",
        "Semulator"     :  "SOFTBANK",
        "Vemulator"     :  "SOFTBANK",
        "PDXGW"         :  "WILLCOM",
        "DDIPOCKET"     :  "WILLCOM",
        "WILLCOM"       :  "WILLCOM",
    }

    for l in _ua_dict.keys():
        if ua.find(l) > -1:
            return (True, _ua_dict[l], "MOBILE", "JP")

    return (False, None, None, None)


def is_mobile_email_domain(domain):
    """
    Checks if a domain name is a valid keitai email domain

    @type domain: string
    @param domain: The domain name
    @rtype: boolean
    @return: True if domain is a valid keitai email domain, False otherwise
    """

    dictionary_of_domains = get_mobile_email_domains()
    for carrier in dictionary_of_domains.keys():
        domain_list = dictionary_of_domains[carrier]
        if domain in domain_list:
            return True

    return False


def is_mobile_phone_number(mn):
    """
    Verifies that a number is a valid mobile phone number

    @type domain: string
    @param domain: The number, without hyphens
    @rtype: boolean
    @return: True if number is a valid number, False otherwise
    """

    _LENGTH = 11
    _VALID_PREFIXES = ( "090", "080", "070", )

    if len(mn) != _LENGTH:
        return False

    if mn[:3] in _VALID_PREFIXES:
        return True

    return False


if __name__ == '__main__':

    # TEST

    print is_mobile_ip("210.153.84.16")
    print is_mobile_ip("122.212.11.154")
    print is_mobile_ip("123.108.237.29")
    print is_mobile_ip("210.153.84.1")
    print is_mobile_ip("210.136.161.98")
    print is_mobile_ip("210.153.86.16")
    print is_mobile_ip("210.153.86.13")
    print is_mobile_ip("210.153.84.11")
    print is_mobile_ip("121.111.231.74")

    ua = "SoftBank/1.0/911T/TJ002/SN354018016346748 Browser/NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1"
    print is_mobile_ua(ua)
    ua = "DoCoMo/2.0 SH901iS(c100;TB;W24H12)"
    print is_mobile_ua(ua)
    ua = "Vodafone/1.0/V905SH/SHJ002 Browser/VF-NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1"
    print is_mobile_ua(ua)
    ua = "KDDI-TS3M UP.Browser/6.2_7.2.7.1.K.2.225 (GUI) MMP/2.0"
    print is_mobile_ua(ua)

    print 'gmail.com is mobile', is_mobile_email_domain('gmail.com')
    print 'ezweb.ne.jp is mobile', is_mobile_email_domain('ezweb.ne.jp')
    print 'emnet.ne.jp is mobile', is_mobile_email_domain('emnet.ne.jp')
    print 'docomo.ne.jp is mobile', is_mobile_email_domain('docomo.ne.jp')
    print 'yahoo.com is mobile', is_mobile_email_domain('yahoo.com')

    print "N", is_mobile_phone_number('09012345678')
    print "N", is_mobile_phone_number('08012345678')
    print "N", is_mobile_phone_number('07012345678')
    print "N", is_mobile_phone_number('03321456434')
    print "N", is_mobile_phone_number('0701234567')
    print "N", is_mobile_phone_number('090-3214-7865')
    print "N", is_mobile_phone_number('0903214786556')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Affandi Mustapa <affandi@marimore.co.jp>
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
These are views for Paypal IPN listener
"""

__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

from django.http import HttpResponse, HttpResponseServerError
from django.shortcuts import HttpResponse, render_to_response, get_object_or_404
from django.conf import settings
import urllib, urllib2

from mamopublic.utils.helpers import log_syslog
from mamopublic.utils.helpers import import_object
from mamopublic.dev.debug import logger

from models import PaypalPayments
from paypalapi import PaypalAPI


def ipn(request):
    """
    These are the required variables in settings.py:

    PAYPAL_SELLER_EMAIL
    PAYPAL_PROD_FLAG
    PAYPAL_PN_FUNCTION
    PAYPAL_LOG_FILE
    """

    if request.method == "POST":

        # Before we do anything, set the encoding for the request
        request.encoding = request.POST['charset']

        try:
            ipn_data = request.POST.copy()

            log_string = "NOTICE: Received IPN from Paypal txn_id:%s" % (ipn_data.get('txn_id', ""))
            log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")

            # Before we do anything else, log the IPN data into the log file
            # It is important to have the original untouched IPN data logged first for debugging
            logger("%s" % (ipn_data), settings.PAYPAL_LOG_FILE)

            # PAYPAL_SELLER_EMAIL is critical to have, so I purposely do not check for exceptions here
            # let the errors come out noisily if we don't have this in settings.py
            paypal_seller_email = settings.PAYPAL_SELLER_EMAIL
            try:
                paypal_prod_flag = settings.PAYPAL_PROD_FLAG
            except:
                paypal_prod_flag = True

            config = {}
            config['paypal_prod_flag'] = paypal_prod_flag
            config['paypal_seller_email'] = paypal_seller_email

            ppapi_object = PaypalAPI(ipn_data, config)

            errcode = ppapi_object.verify_ipn()

            if errcode == -1:
                # Return OK and do not process further
                log_string = "ERROR: IPN from Paypal failed callback verification txn_id:%s" \
                             % (ipn_data.get('txn_id', ""))
                log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")
                return HttpResponse("OK")

            elif errcode == -100:
                # Connecting to Paypal caused an IO Error. Ask Paypal to resend
                log_string = "ERROR: Paypal verification IOError txn_id:%s" % (ipn_data.get('txn_id', ""))
                log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")
                return HttpResponseServerError()

            if not ppapi_object.check_seller_email():
                # Return OK and do not process further
                log_string = "ERROR: IPN from Paypal had invalid seller email txn_id:%s" \
                             % (ipn_data.get('txn_id', ""))
                log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")
                return HttpResponse("OK")

            if PaypalPayments.objects.filter(txn_id=ipn_data.get('txn_id', ""), payment_status="Completed").count() != 0:
                log_string = "ERROR: IPN from Paypal is already complete txn_id:%s" % (ipn_data.get('txn_id', ""))
                log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")
                return HttpResponse("OK")

            txn_type = ipn_data.get('txn_type', "").lower().strip()

            # ipn is verified. Ok, next we must:
            # 1. Write ipn data to log file first
            # 2. Process ipn data againts payment function
            # 3. If payment function is ok, add data to models

            ############# HOOK ##################
            payment_process_function = import_object(settings.PAYPAL_PN_FUNCTION)
            errcode = payment_process_function(ipn_data)
            ############# HOOK ##################

            if errcode == 111:
                # Ask paypal to send the IPN again later
                return HttpResponseServerError()
            elif errcode != 0:
                log_string = "ERROR: Paypal failed payment processing txn_id:%s errcode:%s" \
                             % (ipn_data.get('txn_id', ""), errcode)
                log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")
                return HttpResponse("OK")
            else:
                # errcode == 0
                log_string = "NOTICE: Paypal processing success txn_id:%s" % (ipn_data.get('txn_id', ""))
                log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")

            try:
                # treat transaction ids without txn_id as a new transaction; 
                # subscription signups do not have txn_id
                txn_id = ipn_data.get('txn_id', "")
                if txn_id == "":
                    p1 = PaypalPayments()
                else:
                    p1 = PaypalPayments.objects.get(txn_id=txn_id)
            except:
                p1 = PaypalPayments()

            for index,item in enumerate(ipn_data):                
                p1.__dict__[item] =  ipn_data[item]
            p1.save()

        except Exception, e:
            # Ask paypal to send the IPN again later
            log_string = "ERROR: Exception caught processing txn_id:%s exception:%s" \
                         % (ipn_data.get('txn_id', ""), e)
            log_syslog("paypal.views", log_string, facility="LOG_LOCAL2")
            return HttpResponseServerError()

        return HttpResponse("OK")

    else:
        # Not a POST request. Ignore
        return HttpResponse("OK")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module defines web utilities
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"


import urllib2, urllib

def get_http_basic_auth_opener(url, username, passwd, realm=None):
    """
    Creates and returns OpenerDirector object to access basic authentication protected websites

    You can then use the OpenerDirector to open websites, i.e opendirector.open() as in
    urllib2.open() function.
    
    @type url: string
    @param url: The URL of the page you're trying to access
    @type username: string
    @param username: The URL of the page you're trying to access
    @type passwd: string
    @param passwd: The password for basic authentication
    @type realm: string, defaults to None
    @param realm: The realm you want to access. By default is None (uses default realm)
    @return: an OpenerDirector object
    """

    passwd_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
    passwd_manager.add_password(realm, url, username, passwd)
    auth_handler = urllib2.HTTPBasicAuthHandler(passwd_manager)
    return urllib2.build_opener(auth_handler)

if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Part of this setup script was taken from Django-1.1 setup.py. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Setup script for mamo package
"""

__author__      = "Iqbal Abdullah <iqbal@marimore.co.jp>"
__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

from distutils.core import setup, Extension
from distutils.command.install import INSTALL_SCHEMES
import os

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
mamo_dir = 'mamopublic'

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

for dirpath, dirnames, filenames in os.walk(mamo_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

mamonetx = Extension('mamonetx',
            define_macros = [('MAJOR_VERSION', '1'),('MINOR_VERSION', '0')],
            include_dirs = ['/usr/include', '/usr/local/include'],
            libraries = ['spf'],
            library_dirs = ['/usr/lib', '/home/local/lib'],
            sources = ['ext/mamonetx/netutil.cc'])

setup (
        name='mamopublic',
        version='1.9',
        description='MAMO Libraries',
        long_description='Common libraries being used in python packages for MARIMORE',
        author='MARIMORE Inc.',
        author_email='engineering@marimore.co.jp',
        url='https://github.com/marimore/mamopublic/',
        packages=packages,
        package_data={
            mamo_dir: ['data/ip.dat'],
        },
        ext_modules = [mamonetx],
      )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009, MARIMORE Inc Tokyo, Japan.
# Contributed by 
#       Affandi Mustapa <affandi@marimore.co.jp>
#       Iqbal Abdullah <iqbal@marimore.co.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#   *   Redistributions of source code must retain the above copyright notice, 
#       this list of conditions and the following disclaimer.
#   *   Redistributions in binary form must reproduce the above copyright notice, 
#       this list of conditions and the following disclaimer in the documentation 
#       and/or other materials provided with the distribution.
#   *   Neither the name of the MARIMORE Inc nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
These are models for Paypal billing
"""

__date__        = "$LastChangedDate$"
__version__     = "$LastChangedRevision$"

from django.db import models

class PaypalPayments(models.Model):

    # Transaction information
    # No txn_id for subscriptions; Make blank=True
    txn_id = models.CharField(max_length=255, blank=True)
    txn_type = models.CharField(max_length=255)
    parent_txn_id = models.CharField(max_length=255)        # the parent id for refund, reversal or canceled transactions
    notify_version = models.CharField(max_length=255)
    test_ipn = models.CharField(max_length=255)
    resend = models.CharField(max_length=255)
    charset = models.CharField(max_length=255)
    verify_sign = models.CharField(max_length=255)

    # Advanced and custom information		
    custom = models.CharField(max_length=255, blank=True)

    # Merchant information		
    receiver_email = models.CharField(max_length=255)
    receiver_id = models.CharField(max_length=255, blank=True)
    residence_country = models.CharField(max_length=2)

    # Payment information
    payment_type = models.CharField(max_length=255)
    payment_date = models.CharField(max_length=255)
    payment_status = models.CharField(max_length=255)
    invoice = models.CharField(max_length=255, blank=True)
    memo = models.CharField(max_length=255, blank=True)

    # Product information
    item_number = models.CharField(max_length=255, blank=True)

    # Payment financials
    mc_currency = models.CharField(max_length=255, blank=True)
    mc_gross = models.DecimalField(max_digits=12, decimal_places=3, null=True)     # 1 billion with 3 digits
    mc_fee = models.DecimalField(max_digits=12, decimal_places=3, null=True)       # 1 billion with 3 digits
                    
    #Shipping and handling
    mc_shipping = models.DecimalField(max_digits=12, decimal_places=3, null=True)  # 1 billion with 3 digits
    tax = models.DecimalField(max_digits=12, decimal_places=3, null=True)          # 1 billion with 3 digits

    #Refunds/reversals		
    reason_code = models.CharField(max_length=255)
                   
    #Pending payments
    pending_reason = models.CharField(max_length=255, blank=True)
                    
    #Buyer information
    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)
    payer_status = models.CharField(max_length=255)
    payer_email = models.CharField(max_length=255)
    payer_id = models.CharField(max_length=255)

    address_status = models.CharField(max_length=255, blank=True)
    address_country = models.CharField(max_length=255, blank=True)
    address_city = models.CharField(max_length=255, blank=True)
    address_country_code = models.CharField(max_length=2, blank=True)
    address_name = models.CharField(max_length=255, blank=True)
    address_state = models.CharField(max_length=255, blank=True)
    address_street = models.CharField(max_length=255, blank=True)
    address_zip = models.CharField(max_length=255, blank=True)
    contact_phone = models.CharField(max_length=255, blank=True)

    #Recurring payments
    subscr_id = models.CharField(max_length=255, blank=True)

    #Django custom: Time you received this ipn for the first time
    ipn_received_time     = models.DateTimeField(auto_now=False, auto_now_add=True)
    #Django custom: Time you received an status change for this ipn
    ipn_changed_time     = models.DateTimeField(auto_now=True, auto_now_add=True)

    def __unicode__(self):
        return self.txn_id
